def ratios_analysis(ratios, rotation = False):

    '''
    Graphically gives an overview of the various ratios items sorted from low to high.
    When the ratios DataFrame features multiple companies, you are able to visually 
    see the difference between the companies. 
        
    Parameters
    ----------
    ratios        : DataFrame
                    The data created with the ratios() function.

    rotation      : boolean
                    Default on False. Rotates xlabels vertically when True.
        
    Returns
    -------
    The following graphs:
        pe trailing
        pe forward
        peg ratio
        price to sales ratio
        price to book ratio
        book value per share
        return on assets (ROA)
        return on equity (ROE)
        profit margin
        beta
        current ratio
        debt to equity ratio
    '''

    import numpy as np
    import pandas as pd 
    import matplotlib.pyplot as plt

    ratios = ratios.T
    
    pe_trailing = pd.DataFrame(index=ratios.index)
    pe_forward = pd.DataFrame(index=ratios.index)
    peg_ratio = pd.DataFrame(index=ratios.index)
    price_sales = pd.DataFrame(index=ratios.index)
    price_book = pd.DataFrame(index=ratios.index)
    book_value_per_share = pd.DataFrame(index=ratios.index)
    
    return_on_assets = pd.DataFrame(index=ratios.index)
    return_on_equity = pd.DataFrame(index=ratios.index)
    profit_margin = pd.DataFrame(index=ratios.index)
    beta = pd.DataFrame(index=ratios.index)
    
    current_ratio = pd.DataFrame(index=ratios.index)
    debt_to_equity = pd.DataFrame(index=ratios.index)
    
    pe_trailing = ratios['Trailing P/E']
    pe_forward = ratios['Forward P/E 1']
    peg_ratio = ratios['PEG Ratio (5 yr expected) 1']
    price_sales = ratios['Price/Sales (ttm)']
    price_book = ratios['Price/Book (mrq)']
    book_value_per_share = ratios['Book Value Per Share (mrq)']
    return_on_assets = ratios['Return on Assets (ttm)']
    return_on_equity = ratios['Return on Equity (ttm)']
    profit_margin = ratios['Profit Margin']
    beta = ratios['Beta (3Y Monthly)']
    current_ratio = ratios['Current Ratio (mrq)']
    debt_to_equity = ratios['Total Debt/Equity (mrq)']
    
    if rotation == False:
        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        pe_trailing.sort_values().plot.bar(ax=axes[0,0], rot=0).set_title('P/E Trailing')    
        pe_forward.sort_values().plot.bar(ax=axes[0,1], rot=0).set_title('P/E Forward')
        peg_ratio.sort_values().plot.bar(ax=axes[1,0], rot=0).set_title('PEG Ratio')
        price_sales.sort_values().plot.bar(ax=axes[1,1], rot=0).set_title('Price/Sales')
        plt.show()

        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        price_book.sort_values().plot.bar(ax=axes[0,0], rot=0).set_title('Price/Book')
        book_value_per_share.sort_values().plot.bar(ax=axes[0,1], rot=0).set_title('Book Value per Share')
        return_on_assets.sort_values().plot.bar(ax=axes[1,0], rot=0).set_title('Return on Assets (ROA)')
        return_on_equity.sort_values().plot.bar(ax=axes[1,1], rot=0).set_title('Return on Equity (ROE)')
        plt.show()

        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        profit_margin.sort_values().plot.bar(ax=axes[0,0], rot=0).set_title('Net Profit Margin')
        beta.sort_values().plot.bar(ax=axes[0,1], rot=0).set_title('Beta')
        current_ratio.sort_values().plot.bar(ax=axes[1,0], rot=0).set_title('Current Ratio')
        debt_to_equity.sort_values().plot.bar(ax=axes[1,1], rot=0).set_title('Debt to Equity')
        
        
    else:
        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        pe_trailing.sort_values().plot.bar(ax=axes[0,0]).set_title('P/E Trailing')    
        pe_forward.sort_values().plot.bar(ax=axes[0,1]).set_title('P/E Forward')
        peg_ratio.sort_values().plot.bar(ax=axes[1,0]).set_title('PEG Ratio')
        price_sales.sort_values().plot.bar(ax=axes[1,1]).set_title('Price/Sales')
        plt.show()

        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        price_book.sort_values().plot.bar(ax=axes[0,0]).set_title('Price/Book')
        book_value_per_share.sort_values().plot.bar(ax=axes[0,1]).set_title('Book Value per Share')
        return_on_assets.sort_values().plot.bar(ax=axes[1,0]).set_title('Return on Assets (ROA)')
        return_on_equity.sort_values().plot.bar(ax=axes[1,1]).set_title('Return on Equity (ROE)')
        plt.show()

        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        profit_margin.sort_values().plot.bar(ax=axes[0,0]).set_title('Net Profit Margin')
        beta.sort_values().plot.bar(ax=axes[0,1]).set_title('Beta')
        current_ratio.sort_values().plot.bar(ax=axes[1,0]).set_title('Current Ratio')
        debt_to_equity.sort_values().plot.bar(ax=axes[1,1]).set_title('Debt to Equity')
        
    plt.show()


def balance_sheet(symbol = 0, write_pickle = False, read_pickle = False):

    '''
    Gathers all balance sheet data from Yahoo Finance for the selected companies,
    defined by symbol, formats the data and places it in a DataFrame.
        
    Parameters
    ----------
    symbol        : string or list
                    Company ticker(s) either displayed as a string for one company or as a list
                    when multiple companies are selected.

    write_pickle  : boolean
                    Default on False. Gives the option to write to pickle.

    read_pickle   : boolean
                    Default on False. Gives the option to read the last created pickle.
        
    Returns
    -------
    balance_sheet  : DataFrame
                     All obtained balance sheet data from the selected symbols.
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    import pickle

    if read_pickle == True:
        balance_sheet = pd.read_pickle('balance_sheet')
        return balance_sheet
    
    else:
        balance_sheet = {}
        dummy = 0
    
        if symbol == 0:
            page = requests.get('https://finance.yahoo.com/trending-tickers')
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')
            symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
            print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')

    if type(symbol) == list:
        for s in symbol:
            balance_sheet_url = 'https://finance.yahoo.com/quote/' + s + '/balance-sheet?p=' + s

            page = requests.get(balance_sheet_url)
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')
            data = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0].set_index(0).transpose()

            if len(data) == 0:
                continue

            if 'Volume' in data.columns:
                print('Other entity than a company detected therefore not adding data for: ' + s)

            else:
                for d in data.columns.unique():
                    try:
                        data[d].astype('int64')

                    except ValueError:
                        if '-' in data[d].values:
                            continue

                        elif dummy == 0:
                            index = data['Period Ending']
                            data = data.drop('Period Ending', axis = 1)
                            dummy = 1
                            dates = []

                            for y in index:
                                dates.append(y[-4:])   
                        else:
                            data = data.drop(d, axis=1)

                for c in data.columns: 
                    balance_sheet[s, c] = data[c]
        
    else:
        balance_sheet_url = 'https://finance.yahoo.com/quote/' + symbol + '/balance-sheet?p=' + symbol

        page = requests.get(balance_sheet_url)
        tree = html.fromstring(page.content)
        table = tree.xpath('//table')
        data = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0].set_index(0).transpose()

        if len(data) == 0:
            print('Not data available for' + symbol)

        elif 'Volume' in data.columns:
            print('Other entity than a company detected therefore not adding data for: ' + symbol)

        else:
            for d in data.columns.unique():
                try:
                    data[d].astype('int64')

                except ValueError:
                    if '-' in data[d].values:
                        continue

                    elif dummy == 0:
                        index = data['Period Ending']
                        data = data.drop('Period Ending', axis = 1)
                        dummy = 1
                        dates = []

                        for y in index:
                            dates.append(y[-4:])   
                    else:
                        data = data.drop(d, axis=1)

            for c in data.columns: 
                balance_sheet[symbol, c] = data[c]

    try: 
        balance_sheet = pd.DataFrame(balance_sheet).set_index([dates])
        balance_sheet = balance_sheet.replace('-','0').astype(float).fillna(0)
        balance_sheet = balance_sheet.sort_index()
        
        if write_pickle == True:
            balance_sheet.to_pickle('balance_sheet')

    except ValueError as e:
        print('Could not convert to a DataFrame due to: ', e)

    return balance_sheet


def summary(symbol = 0, write_pickle = False, read_pickle = False):
    
    '''
    Gathers all summary data (homepage) from Yahoo Finance for the selected companies,
    defined by symbol, formats the data and places it in a DataFrame.
        
    Parameters
    ----------
    symbol        : string or list
                    Company ticker(s) either displayed as a string for one company or as a list
                    when multiple companies are selected.

    write_pickle  : boolean
                    Default on False. Gives the option to write to pickle.

    read_pickle   : boolean
                    Default on False. Gives the option to read the last created pickle.
        
    Returns
    -------
    summary       : DataFrame
                     All obtained summary data from the selected symbols.
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    import pickle

    if read_pickle == True:
        summary = pd.read_pickle('summary')
        return summary

    else:
        summary = {}
        dummy = 0
    
        if symbol == 0:
            page = requests.get('https://finance.yahoo.com/trending-tickers')
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')
            symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
            print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')

    if type(symbol) == list:
        for s in symbol:
            summary_url = 'https://finance.yahoo.com/quote/' + s + '?p=' + s

            data = {}

            page = requests.get(summary_url)
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')

            data[0] = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]
            data[1] = pd.read_html(lxml.etree.tostring(table[1], method='html'))[0]

            data[0] = data[0].append(data[1])

            data = data[0].set_index(0)
            data = pd.DataFrame(data[1].fillna(0))

            summary[s] = data[1]
                
    else:
            summary_url = 'https://finance.yahoo.com/quote/' + symbol + '?p=' + symbol

            data = {}

            page = requests.get(summary_url)
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')

            data[0] = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]
            data[1] = pd.read_html(lxml.etree.tostring(table[1], method='html'))[0]

            data[0] = data[0].append(data[1])

            data = data[0].set_index(0)
            data = pd.DataFrame(data[1].fillna(0))

            summary[symbol] = data[1]
        
    summary = pd.DataFrame(summary).fillna(0)
    
    for x in summary.columns:
        for i in range(0,len(summary)):
            if 'M' in str(summary[x][i]):
                try:
                    summary[x][i] = float(summary[x][i].replace('M', '')) * 1000000

                except:
                    continue

            elif 'B' in str(summary[x][i]):
                try:
                    summary[x][i] = float(summary[x][i].replace('B', '')) * 1000000000

                except:
                    continue

            elif '%' in str(summary[x][i]):
                try:
                    summary[x][i] = float(summary[x][i].replace('%', '')) / 100

                except:
                    continue

            else:
                try:
                    summary[x][i] = float(summary[x][i])

                except:
                    continue

    if write_pickle == True:
        summary.to_pickle('summary')
    
    return summary


def rss_feed(symbol = 0, write_pickle = True, read_pickle = False):  

    '''
    Provides latest news from Yahoo Finance for each selected symbol.
        
    Parameters
    ----------
    symbol        : string or list
                    Company ticker(s) either displayed as a string for one company or as a list
                    when multiple companies are selected.

    write_pickle  : boolean
                    Default on False. Gives the option to write to pickle.

    read_pickle   : boolean
                    Default on False. Gives the option to read the last created pickle.
        
    Returns
    -------
    rss_feed      : DataFrame
                    Shows news items from Yahoo Finance.
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import feedparser as fp
    import pickle 

    if read_pickle == True:
        rss_feed = pd.read_pickle('newsfeed')
        return rss_feed
    
    else:
        rss_feed = {}

        if symbol == 0:
            page = requests.get('https://finance.yahoo.com/trending-tickers')
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')
            symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
            print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')
    
    if len(str(symbol)) > 6:
        for s in symbol:
            rss_feed[s,'Title'] = []
            rss_feed[s,'Link'] = []

            feed = fp.parse('http://finance.yahoo.com/rss/headline?s=' + s)


            # RSS Feed Data
            for post in feed['entries']:
                try:
                    rss_feed[s,'Title'].append(post.title)
                    rss_feed[s,'Link'].append(post.link)

                except KeyError:
                    continue
    else:
        rss_feed[symbol,'Title'] = []
        rss_feed[symbol,'Link'] = []

        feed = fp.parse('http://finance.yahoo.com/rss/headline?s=' + symbol)


        # RSS Feed Data
        for post in feed['entries']:
            try:
                rss_feed[symbol,'Title'].append(post.title)
                rss_feed[symbol,'Link'].append(post.link)

            except KeyError:
                continue
    try:
        rss_feed = pd.DataFrame(rss_feed.values(), index=rss_feed.keys()).transpose()
        
        if write_pickle == True:
            rss_feed.to_pickle('newsfeed')

    except ValueError as e:
        print('Could not convert rss_feed to a DataFrame due to: ', e)

    return rss_feed


# Modules
from .balance_sheet import balance_sheet
from .income_statement import income_statement
from .cashflows import cashflows
from .ratios import ratios

from .summary import summary
from .balance_sheet_analysis import balance_sheet_analysis
from .income_statement_analysis import income_statement_analysis
from .cashflows_analysis import cashflows_analysis
from .ratios_analysis import ratios_analysis
from .stock_data import stock_data
from .correlation_matrix import correlation_matrix
from .rss_feed import rss_feed


def correlation_matrix(stock_data, symbol = 0, graph = False):

    '''
    Based on the stock_data() function (when include_returns is set to True),
    calculate ratios and put them in a DataFrame for comparison. Includes the option
    to visually show the correlations.
        
    Parameters
    ----------
    stock_data          : DataFrame
                          Data from the stock_data() function where it checks for the Returns column.
    
    symbol              : string or list
                          Company ticker(s) either displayed as a string for one company or as a list
                          when multiple companies are selected.

    graph               : boolean
                          Default on False. Visually graphs the correlations with Seaborn.
        
    Returns
    -------
    correlation_matrix  : DataFrame
                          Shows correlations between companies.

    if graph = True     : Seaborn Graph
                          Visually shows correlations between companies.
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    import seaborn as sns

    correlation_matrix = pd.DataFrame()
    
    if symbol == 0:
        page = requests.get('https://finance.yahoo.com/trending-tickers')
        tree = html.fromstring(page.content)
        table = tree.xpath('//table')
        symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
        print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')
    
    if type(symbol) == list:
        for s in symbol:
            try:
                correlation_matrix[s] = stock_data['Return', s]
        
            except KeyError:
                print('Could not add ' + s + ' to Matrix')

        if len(correlation_matrix) == 0:
            print('Please set "include_returns" to True in stock_data function.')
            return
        
        correlation_matrix = round(correlation_matrix.corr(), 2)
    
        if graph == True:
            plt.figure(figsize=(20,20))
            sns.heatmap(correlation_matrix,
                    annot = True,
                    cmap = "Reds",
                    annot_kws = {'size':8})

            plt.xticks(rotation = 90)
            plt.yticks(rotation = 0)
            plt.show()
            
        return correlation_matrix
        
    else:
        print('Selected only one company therefore can not create a matrix.')


def stock_data(begin_time, end_time, symbol = 0, write_pickle = False, read_pickle = False, include_returns = False):

    '''
    Provides Stock Data for each company based on a given time period and orders
    the data in a DataFrame. Excludes unavailable company data (i.e. private company)
    and has the option to also provide returns.
        
    Parameters
    ----------
    begin_time      : integer or datetime
                      The starting data, can be an integer (i.e. 2008) or datetime (i.e. '15-07-2008)

    end_time        : integer or datetime
                      The ending data, can be an integer (i.e. 2012) or datetime (i.e. '28-07-2012)
    
    symbol          : string or list
                      Company ticker(s) either displayed as a string for one company or as a list
                      when multiple companies are selected.

    write_pickle    : boolean
                      Default on False. Gives the option to write to pickle.

    read_pickle     : boolean
                      Default on False. Gives the option to read the last created pickle.

    include_returns : boolean
                      Default on False. Gives the option to include returns data.
        
    Returns
    -------
    stockdata       : DataFrame
                      Shows closing values (and returns) per symbol.
    '''
    
    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import pandas_datareader as pdr
    from pandas_datareader._utils import RemoteDataError
    import pickle

    if read_pickle == True:
        stockdata = pd.read_pickle('stockdata')
        return stockdata
    
    else:
        if symbol == 0:
            page = requests.get('https://finance.yahoo.com/trending-tickers')
            tree = html.fromstring(page.content)
            table = tree.xpath('//table')
            symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
            print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')

        stockdata = {}
        
        if type(symbol) == list:
            for s in symbol:
                try:
                    if include_returns == True:
                        stockdata['Close', s] = pdr.DataReader(s, 'yahoo', begin_time, end_time)['Close']
                        stockdata['Return', s] = stockdata['Close', s].pct_change(1).round(4).fillna(0)

                    else:
                        stockdata[s] = pdr.DataReader(s, 'yahoo', begin_time, end_time)['Close']

                except:
                    print('Can not download ' + s + ' ticker data.')
                    
        else:
            try:
                if include_returns == True:
                    stockdata['Close', symbol] = pdr.DataReader(symbol, 'yahoo', begin_time, end_time)['Close']
                    stockdata['Return', symbol] = stockdata['Close', symbol].pct_change(1).round(4).fillna(0)

                else:
                    stockdata[symbol] = pdr.DataReader(symbol, 'yahoo', begin_time, end_time)['Close']

            except:
                print('Can not download ' + symbol + ' ticker data.')

        stockdata = pd.DataFrame(stockdata)
        stockdata = stockdata.fillna(method='ffill', limit=2)
        stockdata = stockdata.fillna(method='bfill', limit=2)
        stockdata = stockdata[~ stockdata.index.duplicated()]
        stockdata = stockdata.reindex(sorted(stockdata.columns), axis=1)

        if write_pickle == True:
            stockdata.to_pickle('stockdata')
                        
        return stockdata


def balance_sheet_analysis(balance_sheet, symbol = 0, log=False):

    '''
    Graphically gives an overview of the various balance sheet items overtime.
    When selecting multiple symbols, you are able to visually see the difference
    between the companies. 
        
    Parameters
    ----------
    balance_sheet : DataFrame
                    The data created with the balance_sheet() function.
    
    symbol        : string or list
                    Company ticker(s) either displayed as a string for one company or as a list
                    when multiple companies are selected.

    log           : boolean
                    Default on False. Gives the option to convert everything in log values.
        
    Returns
    -------
    The following graphs:
        cash
        inventory
        accounts receivable
        short term investments
        property, plant and equipment
        total current assets
        accounts payable
        long term debt
        other current liabilities
        total current liabilities
        common stock
        preferred stock
        retained earnings
        total stockholder equity
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    
    if symbol == 0:
        page = requests.get('https://finance.yahoo.com/trending-tickers')
        tree = html.fromstring(page.content)
        table = tree.xpath('//table')
        symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
        print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')
    
    cash = pd.DataFrame(index=balance_sheet.index)
    inventory = pd.DataFrame(index=balance_sheet.index)
    accounts_receivable = pd.DataFrame(index=balance_sheet.index)
    short_term_investments = pd.DataFrame(index=balance_sheet.index)
    property_plant_equipment = pd.DataFrame(index=balance_sheet.index)
    total_current_assets = pd.DataFrame(index=balance_sheet.index)
    accounts_payable = pd.DataFrame(index=balance_sheet.index)
    long_term_debt = pd.DataFrame(index=balance_sheet.index)
    other_current_liabilities = pd.DataFrame(index=balance_sheet.index)
    total_current_liabilities = pd.DataFrame(index=balance_sheet.index)
    common_stock = pd.DataFrame(index=balance_sheet.index)
    preferred_stock = pd.DataFrame(index=balance_sheet.index)
    retained_earnings = pd.DataFrame(index=balance_sheet.index)
    total_stockholder_equity = pd.DataFrame(index=balance_sheet.index)
    
    
    if type(symbol) == list:
        if log == True:
            import warnings
            warnings.filterwarnings('ignore', category=RuntimeWarning)
            for x in symbol:
                if x in balance_sheet:
                    try: 
                        cash[x] = np.log(balance_sheet[x, 'Cash And Cash Equivalents'])
                        inventory[x] = np.log(balance_sheet[x, 'Inventory'])
                        accounts_receivable[x] = np.log(balance_sheet[x, 'Net Receivables'])
                        short_term_investments[x] = np.log(balance_sheet[x, 'Short Term Investments'])
                        property_plant_equipment[x] = np.log(balance_sheet[x, 'Property Plant and Equipment'])
                        total_current_assets[x] = np.log(balance_sheet[x, 'Total Current Assets'])

                        accounts_payable[x] = np.log(balance_sheet[x, 'Accounts Payable'])
                        long_term_debt[x] = np.log(balance_sheet[x, 'Long Term Debt'])
                        other_current_liabilities[x] = np.log(balance_sheet[x, 'Other Current Liabilities'])
                        total_current_liabilities[x] = np.log(balance_sheet[x, 'Total Current Liabilities'])

                        common_stock[x] = np.log(balance_sheet[x,'Common Stock'])
                        preferred_stock[x] = np.log(balance_sheet[x,'Preferred Stock'])
                        retained_earnings[x] = np.log(balance_sheet[x, 'Retained Earnings'])
                        total_stockholder_equity[x] = np.log(balance_sheet[x, 'Total Stockholder Equity'])
                    
                    except KeyError:
                        continue

            print('Assets')
            fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
            cash.plot.bar(ax=axes[0,0], rot=0).set_title('Cash And Cash Equivalents')
            inventory.plot.bar(ax=axes[0,1], rot=0).set_title('Inventory')
            accounts_receivable.plot.bar(ax=axes[1,0], rot=0).set_title('Accounts Receivables')
            short_term_investments.plot.bar(ax=axes[1,1], rot=0).set_title('Short Term Investments')
            property_plant_equipment.plot.bar(ax=axes[2,0], rot=0).set_title('Property Plant and Equipment')
            total_current_assets.plot.bar(ax=axes[2,1], rot=0).set_title('Total Current Assets')
            plt.show()

            print('Liabilities')
            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            accounts_payable.plot.bar(ax=axes[0,0], rot=0).set_title('Accounts Payable')
            long_term_debt.plot.bar(ax=axes[0,1], rot=0).set_title('Long Term Debt')
            other_current_liabilities.plot.bar(ax=axes[1,0], rot=0).set_title('Other Current Liabilities')
            total_current_liabilities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Current Liabilities')
            plt.show()

            print('Equity')
            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            common_stock.plot.bar(ax=axes[0,0], rot=0).set_title('Common Stock')
            preferred_stock.plot.bar(ax=axes[0,1], rot=0).set_title('Preferred Stock')
            retained_earnings.plot.bar(ax=axes[1,0], rot=0).set_title('Retained Earnings')
            total_stockholder_equity.plot.bar(ax=axes[1,1], rot=0).set_title('Total Stockholder Equity')
            plt.show()
        
        else:
            for x in symbol:
                if x in balance_sheet:
                    try: 
                        cash[x] = balance_sheet[x, 'Cash And Cash Equivalents']
                        inventory[x] = balance_sheet[x, 'Inventory']
                        accounts_receivable[x] = balance_sheet[x, 'Net Receivables']
                        short_term_investments[x] = balance_sheet[x, 'Short Term Investments']
                        property_plant_equipment[x] = balance_sheet[x, 'Property Plant and Equipment']
                        total_current_assets[x] = balance_sheet[x, 'Total Current Assets']

                        accounts_payable[x] = balance_sheet[x, 'Accounts Payable']
                        long_term_debt[x] = balance_sheet[x, 'Long Term Debt']
                        other_current_liabilities[x] = balance_sheet[x, 'Other Current Liabilities']
                        total_current_liabilities[x] = balance_sheet[x, 'Total Current Liabilities']

                        common_stock[x] = balance_sheet[x,'Common Stock']
                        preferred_stock[x] = balance_sheet[x,'Preferred Stock']
                        retained_earnings[x] = balance_sheet[x, 'Retained Earnings']
                        total_stockholder_equity[x] = balance_sheet[x, 'Total Stockholder Equity']
                    
                    except KeyError:
                        continue

            print('Assets')
            fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
            cash.plot.bar(ax=axes[0,0], rot=0).set_title('Cash And Cash Equivalents')
            inventory.plot.bar(ax=axes[0,1], rot=0).set_title('Inventory')
            accounts_receivable.plot.bar(ax=axes[1,0], rot=0).set_title('Accounts Receivables')
            short_term_investments.plot.bar(ax=axes[1,1], rot=0).set_title('Short Term Investments')
            property_plant_equipment.plot.bar(ax=axes[2,0], rot=0).set_title('Property Plant and Equipment')
            total_current_assets.plot.bar(ax=axes[2,1], rot=0).set_title('Total Current Assets')
            plt.show()

            print('Liabilities')
            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            accounts_payable.plot.bar(ax=axes[0,0], rot=0).set_title('Accounts Payable')
            long_term_debt.plot.bar(ax=axes[0,1], rot=0).set_title('Long Term Debt')
            other_current_liabilities.plot.bar(ax=axes[1,0], rot=0).set_title('Other Current Liabilities')
            total_current_liabilities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Current Liabilities')
            plt.show()

            print('Equity')
            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            common_stock.plot.bar(ax=axes[0,0], rot=0).set_title('Common Stock')
            preferred_stock.plot.bar(ax=axes[0,1], rot=0).set_title('Preferred Stock')
            retained_earnings.plot.bar(ax=axes[1,0], rot=0).set_title('Retained Earnings')
            total_stockholder_equity.plot.bar(ax=axes[1,1], rot=0).set_title('Total Stockholder Equity')
            plt.show()

    else:
        cash[symbol] = balance_sheet[symbol,'Cash And Cash Equivalents']
        inventory[symbol] = balance_sheet[symbol,'Inventory']
        accounts_receivable[symbol] = balance_sheet[symbol,'Net Receivables']
        short_term_investments[symbol] = balance_sheet[symbol,'Short Term Investments']
        property_plant_equipment[symbol] = balance_sheet[symbol,'Property Plant and Equipment']
        total_current_assets[symbol] = balance_sheet[symbol,'Total Current Assets']

        accounts_payable[symbol] = balance_sheet[symbol,'Accounts Payable']
        long_term_debt[symbol] = balance_sheet[symbol,'Long Term Debt']
        other_current_liabilities[symbol] = balance_sheet[symbol,'Other Current Liabilities']
        total_current_liabilities[symbol] = balance_sheet[symbol,'Total Current Liabilities']

        common_stock[symbol] = balance_sheet[symbol,'Common Stock']
        preferred_stock[symbol] = balance_sheet[symbol,'Preferred Stock']
        retained_earnings[symbol] = balance_sheet[symbol,'Retained Earnings']
        total_stockholder_equity[symbol] = balance_sheet[symbol,'Total Stockholder Equity']
        
        print('Assets')
        fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
        cash.plot.bar(ax=axes[0,0], rot=0).set_title('Cash And Cash Equivalents')
        inventory.plot.bar(ax=axes[0,1], rot=0).set_title('Inventory')
        accounts_receivable.plot.bar(ax=axes[1,0], rot=0).set_title('Accounts Receivables')
        short_term_investments.plot.bar(ax=axes[1,1], rot=0).set_title('Short Term Investments')
        property_plant_equipment.plot.bar(ax=axes[2,0], rot=0).set_title('Property Plant and Equipment')
        total_current_assets.plot.bar(ax=axes[2,1], rot=0).set_title('Total Current Assets')
        plt.show()
        
        print('Liabilities')
        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        accounts_payable.plot.bar(ax=axes[0,0], rot=0).set_title('Accounts Payable')
        long_term_debt.plot.bar(ax=axes[0,1], rot=0).set_title('Long Term Debt')
        other_current_liabilities.plot.bar(ax=axes[1,0], rot=0).set_title('Other Current Liabilities')
        total_current_liabilities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Current Liabilities')
        plt.show()
        
        print('Equity')
        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        common_stock.plot.bar(ax=axes[0,0], rot=0).set_title('Common Stock')
        preferred_stock.plot.bar(ax=axes[0,1], rot=0).set_title('Preferred Stock')
        retained_earnings.plot.bar(ax=axes[1,0], rot=0).set_title('Retained Earnings')
        total_stockholder_equity.plot.bar(ax=axes[1,1], rot=0).set_title('Total Stockholder Equity')
        plt.show()


def cashflows_analysis(cashflows, symbol=0, log=False):   

    '''
    Graphically gives an overview of the various cashflows items overtime.
    When selecting multiple symbols, you are able to visually see the difference
    between the companies. 
        
    Parameters
    ----------
    cashflows     : DataFrame
                    The data created with the cashflows() function.
    
    symbol        : string or list
                    Company ticker(s) either displayed as a string for one company or as a list
                    when multiple companies are selected.

    log           : boolean
                    Default on False. Gives the option to convert everything in log values.
        
    Returns
    -------
    The following graphs:
        net income
        total operating actitivies
        total investing activities
        total financing activities
        operating profit
    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt

    if symbol == 0:
        page = requests.get('https://finance.yahoo.com/trending-tickers')
        tree = html.fromstring(page.content)
        table = tree.xpath('//table')
        symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
        print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')
        
    net_income = pd.DataFrame(index=cashflows.index)
    total_operating_activities = pd.DataFrame(index=cashflows.index)
    total_investing_activities = pd.DataFrame(index=cashflows.index)
    total_financing_activities = pd.DataFrame(index=cashflows.index)
    operating_profit = pd.DataFrame(index=cashflows.index)
            
    if type(symbol) == list:
        if log == True:
            import warnings
            warnings.filterwarnings('ignore', category=RuntimeWarning)
            for x in symbol:
                if x in cashflows:
                    try:
                        net_income[x] = np.log(cashflows[x,'Net Income'])
                        total_operating_activities[x] = np.log(cashflows[x,'Total Cash Flow From Operating Activities'])
                        total_investing_activities[x] = np.log(cashflows[x, 'Total Cash Flows From Investing Activities'])
                        total_financing_activities[x] = np.log(cashflows[x,'Total Cash Flows From Financing Activities'])
                
                    except KeyError:
                        continue
                    

            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            net_income.plot.bar(ax=axes[0,0], rot=0).set_title('Net Income')
            total_operating_activities.plot.bar(ax=axes[0,1], rot=0).set_title('Total Cash Flow From Operating Activities')
            total_investing_activities.plot.bar(ax=axes[1,0], rot=0).set_title('Total Cash Flows From Investing Activities')
            total_financing_activities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Cash Flows From Financing Activities')
            plt.show()

        else:
            for x in symbol:
                if x in cashflows:
                    try:
                        net_income[x] = cashflows[x,'Net Income']
                        total_operating_activities[x] = cashflows[x,'Total Cash Flow From Operating Activities']
                        total_investing_activities[x] = cashflows[x, 'Total Cash Flows From Investing Activities']
                        total_financing_activities[x] = cashflows[x,'Total Cash Flows From Financing Activities']

                    except KeyError:
                        continue

            fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
            net_income.plot.bar(ax=axes[0,0], rot=0).set_title('Net Income')
            total_operating_activities.plot.bar(ax=axes[0,1], rot=0).set_title('Total Cash Flow From Operating Activities ')
            total_investing_activities.plot.bar(ax=axes[1,0], rot=0).set_title('Total Cash Flows From Investing Activities')
            total_financing_activities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Cash Flows From Financing Activities')
            plt.show()
        
    else:
        net_income[symbol] = cashflows[symbol,'Net Income']
        total_operating_activities[symbol] = cashflows[symbol,'Total Cash Flow From Operating Activities']
        total_investing_activities[symbol] = cashflows[symbol, 'Total Cash Flows From Investing Activities']
        total_financing_activities[symbol] = cashflows[symbol,'Total Cash Flows From Financing Activities']

        fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(15,15))
        net_income.plot.bar(ax=axes[0,0], rot=0).set_title('Net Income')
        total_operating_activities.plot.bar(ax=axes[0,1], rot=0).set_title('Total Cash Flow From Operating Activities ')
        total_investing_activities.plot.bar(ax=axes[1,0], rot=0).set_title('Total Cash Flows From Investing Activities')
        total_financing_activities.plot.bar(ax=axes[1,1], rot=0).set_title('Total Cash Flows From Financing Activities')
        plt.show()


def income_statement_analysis(income_statement, symbol = 0, log=False):
    
    '''
    Graphically gives an overview of the various income statement items overtime.
    When selecting multiple symbols, you are able to visually see the difference
    between the companies. 
        
    Parameters
    ----------
    income_statement : DataFrame
                       The data created with the cashflows() function.
    
    symbol           : string or list
                       Company ticker(s) either displayed as a string for one company or as a list
                       when multiple companies are selected.

    log              : boolean
                       Default on False. Gives the option to convert everything in log values.
        
    Returns
    -------
    The following graphs:
        total revenue
        cost of revenue
        general expenses
        research development
        operating profit
        net income

    '''

    import lxml
    from lxml import html
    import requests
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    
    if symbol == 0:
        page = requests.get('https://finance.yahoo.com/trending-tickers')
        tree = html.fromstring(page.content)
        table = tree.xpath('//table')
        symbol = pd.read_html(lxml.etree.tostring(table[0], method='html'))[0]['Symbol'].to_list()
        print('No input is given thus using the Trending Tickers from Yahoo Finance: https://finance.yahoo.com/trending-tickers')
        
    total_revenue = pd.DataFrame(index=income_statement.index)
    cost_of_revenue = pd.DataFrame(index=income_statement.index)
    general_expenses = pd.DataFrame(index=income_statement.index)
    research_development = pd.DataFrame(index=income_statement.index)
    operating_profit = pd.DataFrame(index=income_statement.index)
    net_income = pd.DataFrame(index=income_statement.index)
        
    
    if type(symbol) == list:
        if log == True:
            import warnings
            warnings.filterwarnings('ignore', category=RuntimeWarning)
            for x in symbol:
                if x in income_statement:
                    try:
                        total_revenue[x] = np.log(income_statement[x,'Total Revenue'])
                        cost_of_revenue[x] = np.log(income_statement[x,'Cost of Revenue'])
                        general_expenses[x] = np.log(income_statement[x, 'Selling General and Administrative'])
                        research_development[x] = np.log(income_statement[x,'Research Development'])
                        operating_profit[x] = np.log(income_statement[x,'Operating Income or Loss'])
                        net_income[x] = np.log(income_statement[x,'Net Income From Continuing Ops'])
                
                    except KeyError:
                        continue
                    

            fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
            total_revenue.plot.bar(ax=axes[0,0], rot=0).set_title('Total Revenue')
            cost_of_revenue.plot.bar(ax=axes[0,1], rot=0).set_title('Cost of Revenue (COGS)')
            general_expenses.plot.bar(ax=axes[1,0], rot=0).set_title('Selling, General and Administrative Expenses (SG&A)')
            research_development.plot.bar(ax=axes[1,1], rot=0).set_title('Research Development')
            operating_profit.plot.bar(ax=axes[2,0], rot=0).set_title('Operating Income or Loss')
            net_income.plot.bar(ax=axes[2,1], rot=0).set_title('Net Income From Continuing Ops')
            plt.show()

        else:
            for x in symbol:
                if x in income_statement:
                    try:
                        total_revenue[x] = income_statement[x,'Total Revenue']
                        cost_of_revenue[x] = income_statement[x,'Cost of Revenue']
                        general_expenses[x] = income_statement[x, 'Selling General and Administrative']
                        research_development[x] = income_statement[x,'Research Development']
                        operating_profit[x] = income_statement[x,'Operating Income or Loss']
                        net_income[x] = income_statement[x,'Net Income From Continuing Ops']

                    except KeyError:
                        continue

            fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
            total_revenue.plot.bar(ax=axes[0,0], rot=0).set_title('Total Revenue')
            cost_of_revenue.plot.bar(ax=axes[0,1], rot=0).set_title('Cost of Revenue (COGS)')
            general_expenses.plot.bar(ax=axes[1,0], rot=0).set_title('Selling, General and Administrative Expenses (SG&A)')
            research_development.plot.bar(ax=axes[1,1], rot=0).set_title('Research Development')
            operating_profit.plot.bar(ax=axes[2,0], rot=0).set_title('Operating Income or Loss')
            net_income.plot.bar(ax=axes[2,1], rot=0).set_title('Net Income From Continuing Ops')
            plt.show()
        
    else:
        total_revenue[symbol] = income_statement[symbol,'Total Revenue']
        cost_of_revenue[symbol] = income_statement[symbol,'Cost of Revenue']
        general_expenses[symbol] = income_statement[symbol, 'Selling General and Administrative']
        research_development[symbol] = income_statement[symbol,'Research Development']
        operating_profit[symbol] = income_statement[symbol,'Operating Income or Loss']
        net_income[symbol] = income_statement[symbol,'Net Income From Continuing Ops']

        fig, axes = plt.subplots(nrows=3, ncols=2, figsize=(15,15))
        total_revenue.plot.bar(ax=axes[0,0], rot=0).set_title('Total Revenue')
        cost_of_revenue.plot.bar(ax=axes[0,1], rot=0).set_title('Cost of Revenue (COGS)')
        general_expenses.plot.bar(ax=axes[1,0], rot=0).set_title('Selling, General and Administrative Expenses (SG&A)')
        research_development.plot.bar(ax=axes[1,1], rot=0).set_title('Research Development')
        operating_profit.plot.bar(ax=axes[2,0], rot=0).set_title('Operating Income or Loss')
        net_income.plot.bar(ax=axes[2,1], rot=0).set_title('Net Income From Continuing Ops')
        plt.show()